@@ -2,7 +2,7 @@
 #
 #                          Frozen-Bubble
 #
-# Copyright (c) 2000 - 2006 The Frozen-Bubble Team
+# Copyright (c) 2000 - 2008 The Frozen-Bubble Team
 #
 # Originally sponsored by Mandriva <http://www.mandriva.com/>
 #
@@ -24,19 +24,17 @@
 package fb_stuff;
 
 use fb_c_stuff;
+use fb_config;
 use Locale::gettext;
-use vars qw(@ISA @EXPORT $FPATH $FLPATH $FBLEVELS $colourblind %POS_1P %POS_2P %POS_MP $BUBBLE_SIZE $ROW_SIZE
+use vars qw(@ISA @EXPORT $FPATH $FLPATH $FBHOME $FBLEVELS $colourblind %POS_1P %POS_2P %POS_MP $BUBBLE_SIZE $ROW_SIZE
             $PI $version);
 @ISA = qw(Exporter);
-@EXPORT = qw($version $FPATH $FLPATH $colourblind $FBLEVELS %POS_1P %POS_2P %POS_MP $BUBBLE_SIZE $ROW_SIZE
+@EXPORT = qw($version $FPATH $FLPATH $colourblind $FBHOME $FBLEVELS %POS_1P %POS_2P %POS_MP $BUBBLE_SIZE $ROW_SIZE
              $PI cat_ member difference2 any every even odd sqr to_bool to_int if_ chomp_
              fold_left output append_to_file min max backtrace basename cp_af all partition ssort
-             sum put_in_hash mapn mapn_ fastuniq deep_copy stringchars t);
+             sum put_in_hash mapn mapn_ before_leaving fastuniq deep_copy stringchars t dbgnet);
 
-$version = '2.1.0';
-
-$FPATH = '@DATADIR@/frozen-bubble';
-$FLPATH = '@LIBDIR@/frozen-bubble';
+$version = '2.2.0';
 
 %POS_1P = ( p1 => { left_limit => 190, right_limit => 446, top_limit => 44, 'initial_bubble_y' => 390,
                     canon => { x => 268, 'y' => 356 },
@@ -45,7 +43,9 @@ $FLPATH = '@LIBDIR@/frozen-bubble';
                     next_bubble => { x => 112, 'y' => 440 },
                     on_top_next_relpos => { x => -3, 'y' => -3 },
                     hurry => { x => 10, 'y' => 265 },
-                    scores => { x => 74, 'y' => 103 } },
+                    scores => { x => 74, 'y' => 103 },
+                    progress => { x => 447, 'y' => 28},
+                  },
 	    centerpanel => { x => 149, 'y' => 190 },
 	    pause_clip => { x => 263, 'y' => 212 },
             compressor_xpos => 318,
@@ -62,6 +62,7 @@ $FLPATH = '@LIBDIR@/frozen-bubble';
                     scores => { x => 160, 'y' => 11 },
                     chatting => { x => 70, 'y' => 465 },
                     left => { x => 30, 'y' => 40 },
+                    progress => { x => 287, 'y' => 28 },
                   },
 	    p1 => { left_limit => 354, right_limit => 610, top_limit => 40, 'initial_bubble_y' => 390,
                     canon => { x => 432, 'y' => 356 },
@@ -73,6 +74,7 @@ $FLPATH = '@LIBDIR@/frozen-bubble';
                     malus => { x => 331, 'y' => 402 },
                     scores => { x => 480, 'y' => 11 },
                     chatting => { x => 390, 'y' => 442 },
+                    progress => { x => 345, 'y' => 28 },
                   },
 	    centerpanel => { x => 153, 'y' => 190 },
 	  );
@@ -89,6 +91,7 @@ $POS_2P{rp1} = $POS_2P{p2};  #- in net/lan 2p mode, use bigger graphics and posi
                     scores => { x => 320, 'y' => 12 },
                     chatting => { x => 215, 'y' => 442 },
                     attackme => { x => 185, 'y' => 448 },
+                    progress => { x => 447, 'y' => 28 },
                   },
             rp1 => { left_limit => 20, right_limit => 148, top_limit => 19, 'initial_bubble_y' => 192,
                      canon => { x => 59, 'y' => 175 },
@@ -102,6 +105,7 @@ $POS_2P{rp1} = $POS_2P{p2};  #- in net/lan 2p mode, use bigger graphics and posi
                      chatting => { x => 5, 'y' => 230 },
                      left => { x => 19, 'y' => 17 },
                      attack => { x => 25, 'y' => 213 },
+                     progress => { x => 149, 'y' => 28 },
                    },
             rp2 => { left_limit => 492, right_limit => 620, top_limit => 19, 'initial_bubble_y' => 192,
                      canon => { x => 531, 'y' => 175 },
@@ -115,6 +119,7 @@ $POS_2P{rp1} = $POS_2P{p2};  #- in net/lan 2p mode, use bigger graphics and posi
                      chatting => { x => 460, 'y' => 230 },
                      left => { x => 491, 'y' => 17 },
                      attack => { x => 496, 'y' => 214 },
+                     progress => { x => 483, 'y' => 28 },
                    },
             rp3 => { left_limit => 20, right_limit => 148, top_limit => 247, 'initial_bubble_y' => 420,
                      canon => { x => 59, 'y' => 404 },
@@ -128,6 +133,7 @@ $POS_2P{rp1} = $POS_2P{p2};  #- in net/lan 2p mode, use bigger graphics and posi
                      chatting => { x => 5, 'y' => 460 },
                      left => { x => 19, 'y' => 245 },
                      attack => { x => 24, 'y' => 442 },
+                     progress => { x => 149, 'y' => 258 },
                    },
             rp4 => { left_limit => 492, right_limit => 620, top_limit => 247, 'initial_bubble_y' => 420,
                      canon => { x => 531, 'y' => 404 },
@@ -141,11 +147,14 @@ $POS_2P{rp1} = $POS_2P{p2};  #- in net/lan 2p mode, use bigger graphics and posi
                      chatting => { x => 460, 'y' => 460 },
                      left => { x => 491, 'y' => 245 },
                      attack => { x => 496, 'y' => 442 },
+                     progress => { x => 483, 'y' => 258 },
                    },
 	    centerpanel => { x => 149, 'y' => 190 },
 	  );
 
-$FBLEVELS = "$ENV{HOME}/.fblevels";
+$FBHOME = "$ENV{HOME}/.frozen-bubble";
+$FBLEVELS = "$FBHOME/levels";
+migrate_resource_files();
 
 $BUBBLE_SIZE = 32;
 $ROW_SIZE = $BUBBLE_SIZE * 7/8;
@@ -280,6 +289,27 @@ sub mapn_(&@) {
     my $f = shift;
     smapn($f, max(map { scalar @$_ } @_), @_);
 }
+sub add_f4before_leaving {
+    my ($f, $b, $name) = @_;
+
+    $fb_stuff::before_leaving::_list->{$b}{$name} = $f;
+    if (!$fb_stuff::before_leaving::_added{$name}) {
+	$fb_stuff::before_leaving::_added{$name} = 1;
+	no strict 'refs';
+	*{"fb_stuff::before_leaving::$name"} = sub {
+	    my $f = $fb_stuff::before_leaving::_list->{$_[0]}{$name} or die '';
+	    $name eq 'DESTROY' and delete $fb_stuff::before_leaving::_list->{$_[0]};
+	    &$f;
+	};
+    }
+}
+#- ! the functions are not called in the order wanted, in case of multiple before_leaving :(
+sub before_leaving(&) {
+    my ($f) = @_;
+    my $b = bless {}, 'fb_stuff::before_leaving';
+    add_f4before_leaving($f, $b, 'DESTROY');
+    $b;
+}
 # -=-=---=-=---=-=---=-=---=-=---=-=---=-=---=-=---=-=---=-=---=-=---=-=---=-=---=-=---=-=--
 
 #- it doesn't keep ordering (but I don't care)
@@ -306,3 +336,25 @@ sub t {
     my ($fmt, @args) = @_;
     return sprintf(gettext($fmt), @args);
 }
+
+sub dbgnet {
+    if (0) {
+        print "DBGNET: @_\n";
+    }
+}
+
+sub migrate_resource_files {
+    mkdir $FBHOME;
+    my %files = (
+	"$ENV{HOME}/.fbhighlevelshistory" => "highlevelshistory",
+	"$ENV{HOME}/.fb_records" => "records",
+	"$ENV{HOME}/.fblevels" => "levels",
+	"$ENV{HOME}/.fbhighscores" => "highscores",
+	"$ENV{HOME}/.fbhighscores-mptrain" => "highscores-mptrain",
+	"$ENV{HOME}/.fbrc" => "rc"
+    );
+    foreach my $file (keys %files) {
+        -r $file or next;
+        system "mv '$file' '$FBHOME/$files{$file}'";
+    }
+}
